#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
import shutil
import stat
import uuid

OUTPUT_FOLDER = "output"
GAME_MOD = "neteaseTemplate"
MASTER_MOD = "neteaseTemplateMaster"
SERVICE_MOD = "neteaseTemplateService"
BEHAVIOR_HEADER_UIID = "734c81d9-a954-4419-858c-95ae01b33480"
BEHAVIOR_MODULE_UUID = "eec826bd-135d-400a-9433-6dd2c197d59a"
RESOURCE_HEADER_UUID = "c4fd04c9-9cc5-4e42-ad3d-1cd18ef251b4"
RESOURCE_MODULE_UUID = "4fb2278e-e3af-4d11-8488-8c015bdc49a7"

NEW_UUID_1 = str(uuid.uuid1())
NEW_UUID_2 = str(uuid.uuid1())
NEW_UUID_3 = str(uuid.uuid1())
NEW_UUID_4 = str(uuid.uuid1())

def MakeMod(teamName, modName, modType):
    print "Make Mod TeamName:%s ModName:%s ModName:%s" % (teamName, modName, modType)
    print "uuid1:[%s] uuid2:[%s] uuid3:[%s] uuid4:[%s]" % (NEW_UUID_1, NEW_UUID_2, NEW_UUID_3, NEW_UUID_4)
    folder = os.path.exists(OUTPUT_FOLDER)
    if not folder:
        os.makedirs(OUTPUT_FOLDER)
        print "create output folder:%s" % OUTPUT_FOLDER

    teamName = teamName[0].lower() + teamName[1:]
    destMod = teamName + modName[0].upper() + modName[1:]
    templateMod = ""
    if modType == "game":
        templateMod = GAME_MOD
    elif modType == "master":
        templateMod = MASTER_MOD
        destMod += "Master"
    elif modType == "service":
        templateMod = SERVICE_MOD
        destMod += "Service"
    else:
        print "ERROR: ModType [%s] not in (game,master,service)" % modType
        return

    # 复制模板mod
    def remove_readonly(func, path, _):
        os.chmod(path, stat.S_IWRITE)
        func(path)

    print "copy folder from %s to %s" % (templateMod, os.path.join(OUTPUT_FOLDER,destMod))
    destFolder = os.path.join(OUTPUT_FOLDER, destMod)
    if os.path.exists(destFolder):
        print "remove %s" % destFolder
        shutil.rmtree(destFolder, onerror=remove_readonly)
    shutil.copytree(templateMod, destFolder)

    # 遍历目录重命名
    for path, dirList, fileList in os.walk(destFolder, topdown=False):

        for file in fileList:
            filePath = os.path.join(path, file)

            # 替换文本内容
            with open(filePath) as f:
                contents = f.read()
                f.close()
            
            newContents = replaceContents(contents, teamName, modName)

            if (file == ("manifest.json")) or (file == ("world_behavior_packs.json")) or (file == "world_resource_packs.json"):
                newContents = replaceUUID(newContents)
                print "replace uuid %s" % filePath

            with open(filePath, 'w') as f:
                f.write(newContents)
                f.close()

            # 重命名文件夹
            newFilePath = replaceContents(file, teamName, modName)
            newFilePath = os.path.join(path, newFilePath)
            if newFilePath != filePath:
                print "File rename from %s to %s" % (filePath, newFilePath)
                os.rename(filePath, newFilePath)

        
        for dir in dirList:
            dirPath = os.path.join(path, dir)
            newDirPath = replaceContents(dir, teamName, modName)
            newDirPath = os.path.join(path, newDirPath)
            if newDirPath != dirPath:
                print "Dir rename from %s to %s" % (dirPath, newDirPath)
                os.rename(dirPath, newDirPath)

def replaceContents(origin, teamName, modName):
    ret = origin.replace("netease", teamName)
    ret = ret.replace("template", modName[0].lower() + modName[1:])
    ret = ret.replace("Template", modName[0].upper() + modName[1:])
    return ret

def replaceUUID(contents):
    ret = contents.replace(BEHAVIOR_HEADER_UIID, NEW_UUID_1)
    ret = ret.replace(BEHAVIOR_MODULE_UUID, NEW_UUID_2)
    ret = ret.replace(RESOURCE_HEADER_UUID, NEW_UUID_3)
    ret = ret.replace(RESOURCE_MODULE_UUID, NEW_UUID_4)
    return ret

if __name__ == "__main__":
    # python ModMaker.py teamname modname modtype
    if (len(sys.argv) != 4):
        print "run format: python ModMaker.py teamname modname modtype"
    MakeMod(sys.argv[1], sys.argv[2], sys.argv[3])